unit UnidadMenuPrincipal;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, IniFiles, ShellApi, registry, StdCtrls, ExtCtrls;
type
  //registro para lista de elementos
  PUrlRec = ^TUrlRec;
  TUrlRec = record
    Rep:     String;         // Folder name (folders and links)
    UrlName: String;         // Logical URL name (links only)
    UrlPath: String;         // www URL name (links only)
    Level: Integer;          // Folder level (position in folder tree)
  end;

type
  TformMenuPrincipal = class(TForm)
    Label1: TLabel;
    txtFichero: TEdit;
    bSeleccionar: TButton;
    opAbrir: TCheckBox;
    Button2: TButton;
    bSalir: TButton;
    LWEB: TLabel;
    Panel1: TPanel;
    dlGuardar: TSaveDialog;
    Label2: TLabel;
    procedure LWEBClick2(Sender: TObject);
    procedure bSalirClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure bSeleccionarClick(Sender: TObject);
  private
    { Private declarations }
    carperaFavoritos : string;      //Ruta de la carpeta Favoritos
    listaFavoritos : TList;         //Lista de Favoritos
    listaMarcadores : TStringList;  //Favoritos convertidos en marcadores

    procedure obtenerCarpetaFavoritos;
    procedure buscarURL(carpeta : String; nivel : Integer);
    procedure recorrerListaFavoritos(Idx, PrevIdx: Integer);
    procedure crearFicheroMarcadores;    
    procedure crearDocumentoSuperior;
    procedure crearDocumentoInferior;
    procedure crearEncabezado (UrlRec: PUrlRec);
    procedure crearPie(UrlRec: PUrlRec);
    procedure crearMarcador (UrlRec: PUrlRec);                
  public
    { Public declarations }
  end;

var
  formMenuPrincipal: TformMenuPrincipal;

implementation

{$R *.dfm}

//Busca el directorio de Favoritos de IE
procedure TFormMenuPrincipal.obtenerCarpetaFavoritos;
var
  Registry: TRegistry;
begin
  Registry := TRegistry.Create;
  Registry.RootKey := HKEY_CURRENT_USER;
  Registry.OpenKey('Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders', False);
  carperaFavoritos := Registry.ReadString('Favorites');
  Registry.Free;
end;

// Busca y crea una lista con los ficheros de favoritos encontrados
// este mtodo utiliza modo recursivo para buscar en subcarpetas
procedure TFormMenuPrincipal.buscarURL(carpeta : String; nivel : Integer);
var
  encontrado : Integer;
  busquedaRec: TSearchRec;
  ficheroURL : TIniFile;
  UrlRec: PUrlRec;
begin
  //primer fichero
  encontrado := FindFirst(carpeta + '\*.*', faAnyFile, busquedaRec);
  //buscar el siguiente fichero
  while encontrado = 0 do
  begin
    // para que no coja directorios especiales ('.' y '..')
    if (busquedaRec.Name <> '.') and (busquedaRec.Name <> '..') then
      //cambiamos de directorio
      if (busquedaRec.Attr and faDirectory > 0) then
        begin
          if not (busquedaRec.Attr and faSysFile > 0) then
          begin
            UrlRec := New(PUrlRec);
            UrlRec.Rep := Copy(carpeta + '\' + busquedaRec.Name,
                Length(carperaFavoritos) + 2, Length(carpeta + '\' +
                busquedaRec.Name));
            UrlRec.UrlName := '';
            UrlRec.UrlPath := '';
            UrlRec.Level := nivel;
            listaFavoritos.Add(UrlRec);
          end;
          buscarURL (carpeta + '\' + busquedaRec.Name, nivel + 1);   // Recursin
        end
      else
        begin
          //obtener la extensin del fichero
          if UpperCase(ExtractFileExt(busquedaRec.Name)) = '.URL' then
          begin
            UrlRec := New(PUrlRec);
            UrlRec.Rep := Copy(carpeta, Length(carperaFavoritos) + 2, Length(carpeta));
            UrlRec.UrlName:= Copy(busquedaRec.Name, 0, Length(busquedaRec.Name)-3);
            //Si la URL tiene estructura INI abrimos el fichero y leemos la ruta
            ficheroURL := TIniFile.Create(carpeta + '\' + busquedaRec.Name);
            UrlRec.UrlPath := ficheroURL.ReadString('InternetShortcut', 'URL', 'no path');
            UrlRec.Level := nivel;
            ficheroURL.Free;
            listaFavoritos.Add(UrlRec); //aadimos a la lista de favoritos
          end;
        end;
      //otro fichero
      encontrado := FindNext(busquedaRec);
  end;
end;

//Mtodo recursivo para obtener todas las URLs en sus respectivas carpetas
procedure TFormMenuPrincipal.recorrerListaFavoritos(Idx, PrevIdx: Integer);
var
  UrlRec, PrevUrlRec: PUrlRec;
  X: Integer;
begin
  if Idx < listaFavoritos.Count then
  begin
    UrlRec := listaFavoritos[Idx];
    if PrevIdx = -1 then
      PrevUrlRec := nil
    else
      PrevUrlRec := listaFavoritos[PrevIdx];
    if UrlRec.UrlName = '' then   //Carpeta encontrada (No es una URL)
    begin
      if PrevUrlRec <> nil then
      begin
        X := PrevUrlRec.Level;
        while UrlRec.Level <= X do
        begin
          crearPie(PrevUrlRec);
          Dec(PrevUrlRec.Level);
          Dec(X);
        end;
      end;
      crearEncabezado(UrlRec);
      PrevIdx := Idx;
    end
    else
      crearMarcador (UrlRec);
    recorrerListaFavoritos (Idx+1, PrevIdx);
  end;
end;

//Crear fichero de marcadores con los ficheros encontrados en la carpeta favoritos
procedure TFormMenuPrincipal.crearFicheroMarcadores;
begin
  listaMarcadores := TStringList.Create;
  crearDocumentoSuperior;
  recorrerListaFavoritos (0, -1); //llamada recursiva
  crearDocumentoInferior;
  try
    listaMarcadores.SaveToFile(txtFichero.Text);
  except on EFCreateError do
    begin
      MessageDlg('El nombre de archivo especificado para la exportacin ' +
          'no es correcto.', mtError, [MbOk], 0);
      txtFichero.SetFocus;
    end;
  end;

  listaMarcadores.Free;
  MessageDlg('La carpeta de Favoritos de IE ha sido exportada correctamente ' +
      'al fichero:' + txtFichero.Text, mtInformation, [MbOk], 0);
end;

procedure TFormMenuPrincipal.crearDocumentoSuperior;
begin
  listaMarcadores.Add('FAVORITOS EXPORTADOS ' + DateToStr(date));
  listaMarcadores.Add('');
  listaMarcadores.Add('<TITLE>' + 'Favoritos exportados (AjpdSoft Exportar Favoritos)' + '</TITLE>');
  listaMarcadores.Add('<H1>' + '' + '</H1>');
  listaMarcadores.Add('');
  listaMarcadores.Add('<DL><P>');
end;


procedure TFormMenuPrincipal.crearDocumentoInferior;
begin
  listaMarcadores.Add('<br><br><br>');
  listaMarcadores.Add('Por <a href="http://www.ajpdsoft.com">AjpdSoft Exportar Favoritos</a>');
  listaMarcadores.Add('</DL><P>');
end;

//aade el nombre del ttulo a la seccin
procedure TFormMenuPrincipal.crearEncabezado (UrlRec: PUrlRec);
var
  I, Idx: Integer;
  S: String;
  A: array[0..255] of Char;
begin
  //Cambiar nombre de las sucarpetas
  FillChar(A, SizeOf(A), 0);
  Idx := 1;
  for I := 1 to Length(UrlRec.Rep) do
    if UrlRec.Rep[I] = '\' then
      Idx := I+1;
  for I := Idx to Length(UrlRec.Rep) do
    A[I-Idx] := UrlRec.Rep[I];

  //Aadir nombre al encabezado
  listaMarcadores.Add('');
  for I := 1 to UrlRec.Level do
    S := S + '    ';
  listaMarcadores.Add(S + '<DT><H3>' + A + '</H3>');
  listaMarcadores.Add(S + '<DL><P>');
end;

//Seccin final que contiene los marcadores
procedure TFormMenuPrincipal.crearPie(UrlRec: PUrlRec);
// End section containing bookmarks
var
  I: Integer;
  S: String;
begin
  for I := 1 to UrlRec.Level do
    S := S + '    ';
  listaMarcadores.Add(S + '</DL><P>');
  listaMarcadores.Add('');
end;


procedure TFormMenuPrincipal.crearMarcador (UrlRec: PUrlRec);
var
  I: Integer;
  S: String;
begin
  //Quitamos el punto extra del final del fichero
  Delete(UrlRec.UrlName, Length(UrlRec.UrlName), 1);
  //Unimos la ruta y el nombre
  for I := 1 to UrlRec.Level do
    S := S + '    ';
  listaMarcadores.Add(S + '<DT><A HREF="' + UrlRec.UrlPath + '">' +
              UrlRec.UrlName + '</A>' + '<BR>');
end;


function SortFunction(Item1, Item2: Pointer): Integer;
// Function to sort the list of favorites alphabetically
var
  Rec1, Rec2: PUrlRec;
begin
  Result := 0;
  Rec1 := Item1;
  Rec2 := Item2;
  if Rec1.Rep < Rec2.Rep then Result := -1;
  if Rec1.Rep > Rec2.Rep then Result := 1;
  if Rec1.Rep = Rec2.Rep then
  begin
    if Rec1.UrlName < Rec2.UrlName then Result := -1;
    if Rec1.UrlName > Rec2.UrlName then Result := 1;
    if Rec1.UrlName = Rec2.UrlName then Result := 0;
  end;
end;




procedure TformMenuPrincipal.LWEBClick2(Sender: TObject);
begin
  ShellExecute(Handle, Nil, PChar(LWEB.CAPTION),
      Nil, Nil, SW_SHOWNORMAL);
end;

procedure TformMenuPrincipal.bSalirClick(Sender: TObject);
begin
  close;
end;

procedure TformMenuPrincipal.FormCreate(Sender: TObject);
begin
  txtFichero.Text := ExtractFilePath(Application.ExeName) + 'Favoritos.html';
end;

procedure TformMenuPrincipal.Button2Click(Sender: TObject);
var
  i : Integer;
begin
  listaFavoritos := TList.Create;
  obtenerCarpetaFavoritos;
  if carperaFavoritos <> '' then
  begin
    buscarURL (carperaFavoritos, 1);
    listaFavoritos.Sort(SortFunction);
    crearFicheroMarcadores;
  end;
  for i := 0 to listaFavoritos.Count - 1 do
    Dispose(PUrlRec(listaFavoritos[i]));
  listaFavoritos.Free;
  if opAbrir.Checked then
    if ShellExecute(Application.MainForm.Handle, nil, PChar(txtFichero.Text),
        nil, nil, SW_SHOWNORMAL) <= 32 then
      MessageDlg('No se ha podido iniciar el Explorador de Internet.',
          mtError, [mbOk], 0);
end;

procedure TformMenuPrincipal.bSeleccionarClick(Sender: TObject);
begin
  if dlGuardar.Execute then
    txtFichero.Text := dlGuardar.FileName;
end;

end.
